import { message } from 'antd';
import { routerRedux } from 'dva/router';
import {
  queryCmsUserList,
  createCmsUserItem,
  updateCmsUserItem,
  deleteCmsUserItem,
} from '../services/cmsUser';

export default {
  namespace: 'cmsUser',

  state: {
    list: [],
    pageNo: 1,
    pageSize: 15,
    totalSize: 0,
    currentItem: {},
  },

  effects: {
    *fetchCmsUserList({ payload }, { call, put }) {
      const response = yield call(queryCmsUserList, payload);
      if (response.success) {
        yield put({
          type: 'querySuccess',
          payload: {
            list: response.data.list || [],
            pageSize: response.data.pageSize,
            totalSize: response.data.totalSize,
            pageNo: response.data.pageNo,
          },
        });
      }
    },
    *createCmsUserItem({ payload, state }, { call, put }) {
      const response = yield call(createCmsUserItem, payload);
      if (response.success) {
        message.success('创建系统用户成功');
        yield put(routerRedux.push({
          state,
          pathname: '/system/cms-user/list',
        }));
      }
    },
    *deleteCmsUserItem({ payload, states }, { call, put }) {
      const response = yield call(deleteCmsUserItem, payload);
      if (response.success) {
        message.success('禁用系统用户成功');
        yield put({
          type: 'fetchCmsUserList',
          payload: states.Queryers,
        });
      }
    },
    *recoverCmsUserItem({ payload, states }, { call, put }) {
      const response = yield call(updateCmsUserItem, payload);
      if (response.success) {
        message.success('解禁系统用户成功');
        yield put({
          type: 'fetchCmsUserList',
          payload: states.Queryers,
        });
      }
    },
    *updateCmsUserItem({ payload, states }, { call, put }) {
      const response = yield call(updateCmsUserItem, payload);
      if (response.success) {
        message.success('修改系统用户成功');
        yield put(routerRedux.push({
          pathname: '/system/cms-user/list',
          state: states,
        }));
      }
    },
  },

  reducers: {
    querySuccess(state, action) {
      return {
        ...state,
        ...action.payload,
      };
    },
    fixCurrentItem(state, action) {
      const { currentItem } = state;
      return {
        ...state,
        currentItem: {
          ...currentItem,
          ...action.payload,
        },
      };
    },
    cleanState(state) {
      return {
        ...state,
        list: [],
        currentItem: {},
      };
    },
  },
};
