import { stringify } from 'qs';
import request from '../utils/request';
import { api, Hotax } from '../utils/config';

/**
 * 查询终端用户列表
 * @param params
 * @returns {Promise<Object>}
 */
export async function queryTerminalList(params) {
  const newParams = {
    pageSize: Hotax.PAGE_SIZE,
    ...params,
  };
  return request(`${api.terminal}?${stringify(newParams)}`);
}

/**
 * 创建终端用户
 * @param params
 * @returns {Promise<Object>}
 */
export async function createTerminalItem(params) {
  const options = {
    method: 'POST',
    body: params,
  };
  return request(`${api.terminalItem}`, options);
}

/**
 * 修改终端用户
 * @param params
 * @returns {Promise<Object>}
 */
export async function updateTerminalItem(params) {
  const options = {
    method: 'PUT',
    body: params,
  };
  return request(`${api.terminalItem}`, options);
}

/**
 * 删除终端用户
 * @param id
 * @returns {Promise<Object>}
 */
export async function deleteTerminalItem({ id }) {
  const options = {
    method: 'DELETE',
  };
  return request(`${api.terminalItem}/${id}`, options);
}

/**
 * 查询白名单用户
 * @param params
 * @returns {Promise<Object>}
 */
export async function querySpecialTerminalList(params) {
  const newParams = {
    pageSize: Hotax.PAGE_SIZE,
    ...params,
  };
  return request(`${api.specialTerminal}?${stringify(newParams)}`);
}

/**
 * 查询白名单用户详情
 * @param userId
 * @returns {Promise<Object>}
 */
export async function querySpecialTerminalItem({ userId }) {
  return request(`${api.specialTerminalItem}/${userId}`);
}

/**
 * 创建白名单用户
 * @param params
 * @returns {Promise<Object>}
 */
export async function createSpecialTerminalItem(params) {
  const options = {
    method: 'POST',
    body: params,
  };
  return request(`${api.specialTerminalItem}`, options);
}

/**
 * 修改白名单用户
 * @param params
 * @returns {Promise<Object>}
 */
export async function updateSpecialTerminalItem(params) {
  const options = {
    method: 'PUT',
    body: params,
  };
  return request(`${api.specialTerminalItem}`, options);
}

/**
 * 删除白名单用户
 * @param params
 * @returns {Promise<Object>}
 */
export async function deleteSpecialTerminalItem(params) {
  const options = {
    method: 'DELETE',
    body: params,
  };
  return request(`${api.specialTerminalItem}`, options);
}

/**
 * 解除终端绑定
 * @param id
 * @returns {Promise<Object>}
 */
export async function deviceUnbound({ id }) {
  const options = {
    method: 'DELETE',
  };
  return request(`${api.terminalUnbound}/${id}`, options);
}

/**
 * 查询终端权限列表
 * @param params
 * @returns {Promise<Object>}
 */
export async function queryTerminalAuthList(params) {
  const newParams = {
    pageSize: Hotax.PAGE_SIZE,
    ...params,
  };
  return request(`${api.terminalAuth}?${stringify(newParams)}`);
}

/**
 * 修改用户权限时长
 * @param params
 * @returns {Promise<Object>}
 */
export async function updateTerminalAuth(params) {
  const options = {
    method: 'POST',
    body: params,
  };
  return request(`${api.terminalAuthEdit}`, options);
}

/**
 * 查询终端标签列表
 * @param uid
 * @returns {Promise<Object>}
 */
export async function queryTerminalTagList({ uid }) {
  return request(`${api.userTags}/${uid}`);
}

/**
 * 获取终端标签详情
 * @param userTagId
 * @returns {Promise<Object>}
 */
export async function queryTerminalTagItem({ userTagId }) {
  return request(`${api.userTag}/${userTagId}`);
}

/**
 * 创建终端标签
 * @param params
 * @returns {Promise<Object>}
 */
export async function createTerminalTagItem(params) {
  const options = {
    method: 'POST',
    body: params,
  };
  return request(`${api.userTag}`, options);
}

/**
 * 修改终端标签
 * @param params
 * @returns {Promise<Object>}
 */
export async function updateTerminalTagItem(params) {
  const options = {
    method: 'PUT',
    body: params,
  };
  return request(`${api.userTag}`, options);
}

/**
 * 删除终端标签
 * @param userTagId
 * @returns {Promise<Object>}
 */
export async function deleteTerminalTagItem({ userTagId }) {
  const options = {
    method: 'DELETE',
  };
  return request(`${api.userTag}/${userTagId}`, options);
}

/**
 * 复制渠道标签
 * @param params
 * @returns {Promise<Object>}
 */
export async function copyMerchantTag(params) {
  return request(`${api.userTagCopy}?${stringify(params)}`);
}

/**
 * 查询终端的推荐课程
 * @param params
 * @returns {Promise<Object>}
 */
export async function queryTerminalRecommendCourse({ uid }) {
  return request(`${api.userRecommend}/${uid}`);
}

/**
 * 修改终端的推荐课程
 * @param uid <用户id>
 * @param idList <课程的 idList>
 * @returns {Promise<Object>}
 */
export async function updateTerminalRecommendCourse({ uid, idList }) {
  const options = {
    method: 'PUT',
    body: idList,
  };
  return request(`${api.userRecommend}/${uid}`, options);
}

/**
 * 查询用户登录设备记录
 * @param uid
 * @returns {Promise<Object>}
 */
export async function queryTerminalLoggedDevice({ uid }) {
  return request(`${api.userDevice}/${uid}`);
}
